package ismanevr

import (
	"sync"
)

/*
	Потокобезопасный признак внешнего разрешения манёвра
*/

// IsManevr -- потокобезопасный признак внешнего разрешения манёвра
type IsManevr struct {
	val   bool
	block sync.RWMutex
}

// NewIsManevr -- возвращает новый *IsManevr
func NewIsManevr() *IsManevr {
	return &IsManevr{
		val: true,
	}
}

// Get -- возвращает хранимое состояние
func (sf *IsManevr) Get() bool {
	sf.block.RLock()
	defer sf.block.RUnlock()
	return sf.val
}

// Set -- устанавливает хранимое состояние
func (sf *IsManevr) Set() {
	sf.block.Lock()
	defer sf.block.Unlock()
	sf.val = true
}

// Reset -- сбрасывает хранимое состояние
func (sf *IsManevr) Reset() {
	sf.block.Lock()
	defer sf.block.Unlock()
	sf.val = false
}
